import pytest

import mozjpeg_lossless_optimization


class Test_optimize(object):
    @pytest.fixture
    def image_bytes(self):
        with open("./tests/image.jpg", "rb") as file_:
            return file_.read()

    def test_optimize_returns_bytes(self, image_bytes):
        result = mozjpeg_lossless_optimization.optimize(image_bytes)
        assert type(result) is bytes

    def test_optimize_returns_a_jpeg(self, image_bytes):
        result = mozjpeg_lossless_optimization.optimize(image_bytes)
        assert result.startswith(b"\xff\xd8\xff\xe0\x00\x10JFIF")

    def test_optimize_returns_a_smaller_jpeg(self, image_bytes):
        result = mozjpeg_lossless_optimization.optimize(image_bytes)
        assert len(result) > 0
        assert len(result) < len(image_bytes)

    def test_optimize_raise_error_with_troncated_input_jpeg(self):
        with pytest.raises(ValueError):
            mozjpeg_lossless_optimization.optimize(b"\xff\xd8\xff\xe0\x00\x10JFIF")

    def test_optimize_raise_error_with_invalid_input_data(self):
        with pytest.raises(ValueError):
            mozjpeg_lossless_optimization.optimize(b"foobar")

    def test_optimize_with_copy(self, image_bytes):
        result = mozjpeg_lossless_optimization.optimize(
            image_bytes, copy=mozjpeg_lossless_optimization.COPY_MARKERS.ALL
        )
        assert result.startswith(b"\xff\xd8\xff\xe0\x00\x10JFIF")



import os
from distutils import ccompiler

from cffi import FFI


_ROOT = os.path.abspath(os.path.dirname(__file__))
_C_FILE = os.path.join(_ROOT, "mozjpeg_opti.c")
_H_FILE = os.path.join(_ROOT, "mozjpeg_opti.h")
if ccompiler.get_default_compiler() == "unix":
    _LIBTURBOJPEG_STATIC_LIB = os.path.join(
        _ROOT, "..", "mozjpeg", "build", "libturbojpeg.a"
    )
elif ccompiler.get_default_compiler() == "msvc":
    _LIBTURBOJPEG_STATIC_LIB = os.path.join(
        _ROOT, "..", "mozjpeg", "build", "Release", "turbojpeg-static.lib"
    )

ffibuilder = FFI()
ffibuilder.set_source(
    "mozjpeg_lossless_optimization._mozjpeg_opti",
    open(_C_FILE, "r").read(),
    extra_objects=[_LIBTURBOJPEG_STATIC_LIB],
    include_dirs=[
        _ROOT,
        os.path.join(_ROOT, "..", "mozjpeg"),
        os.path.join(_ROOT, "..", "mozjpeg", "build"),
    ],
)
ffibuilder.cdef(open(_H_FILE, "r").read())


if __name__ == "__main__":
    ffibuilder.compile(verbose=True)



from enum import IntEnum

from ._mozjpeg_opti import lib, ffi


class COPY_MARKERS(IntEnum):
    """Metadata copy options."""

    #: copy no optional markers
    NONE = 0

    #: copy only comment (COM) markers
    COMMENTS = 1

    #: copy all optional markers
    ALL = 2

    #: copy all optional markers except APP2
    ALL_EXCEPT_ICC = 3

    #: copy only ICC profile (APP2) markers
    ICC = 4


def optimize(input_jpeg_bytes, copy=COPY_MARKERS.NONE):
    """Optimize a JPEG.

    :param bytes input_jpeg_bytes: The input JPEG.
    :param COPY copy: Metadata copy options (optional, default ``COPY_MARKERS.NONE``).

    :raises ValueError: If MozJPG was not able to process the input image.

    :rtype: bytes
    :returns: The optimized JPEG.
    """
    output_jpeg_bytes_p = ffi.new("unsigned char**")
    output_jpeg_bytes_p_gc = ffi.gc(
        output_jpeg_bytes_p, lib.mozjpeg_lossless_optimization_free_bytes
    )
    copy_option = COPY_MARKERS(copy).value

    output_jpeg_length = lib.mozjpeg_lossless_optimization(
        input_jpeg_bytes,
        len(input_jpeg_bytes),
        output_jpeg_bytes_p_gc,
        copy_option,
    )

    if output_jpeg_length == 0:
        raise ValueError("MozJPEG was not able to process the input image")

    output_jpeg = ffi.cast("unsigned char*", output_jpeg_bytes_p_gc[0])
    return bytes(ffi.unpack(output_jpeg, output_jpeg_length))



#!/usr/bin/env python
# encoding: UTF-8

import os
import subprocess

from setuptools import setup, find_packages
from setuptools.command.build_ext import build_ext


def _find_msbuild(plat_spec="x64"):
    from setuptools import msvc

    if hasattr(msvc, "msvc14_get_vc_env"):
        vc_env = msvc.msvc14_get_vc_env(plat_spec)
        if "vsinstalldir" in vc_env:
            msbuild_path = os.path.join(
                vc_env["vsinstalldir"], "MSBuild", "Current", "Bin", "MSBuild.exe"
            )
            if os.path.isfile(msbuild_path):
                return msbuild_path

    for path in msvc.EnvironmentInfo(plat_spec).VCTools:
        if "\\VC\\" not in path:
            continue
        msbuild_path = os.path.join(
            path[: path.index("\\VC\\")], "MSBuild", "Current", "Bin", "MSBuild.exe"
        )
        if os.path.isfile(msbuild_path):
            return msbuild_path

    raise Exception("Unable to find any Visual Studio installation")


class CustomBuildExt(build_ext):
    def build_extensions(self):
        if not os.path.isdir("./mozjpeg/build"):
            os.mkdir("./mozjpeg/build")

        os.chdir("./mozjpeg/build")
        cmake_command = [
            "cmake",
            "..",
            "-B",
            ".",
            "-DBUILD_SHARED_LIBS=FALSE",
            "-DPNG_SUPPORTED=FALSE",
            "-DCMAKE_BUILD_TYPE=Release",
        ]

        if self.compiler.compiler_type == "unix":
            os.environ["CFLAGS"] = "%s -fPIC" % os.environ.get("CFLAGS", "")
            subprocess.call(cmake_command)
            subprocess.call(["make"])
        elif self.compiler.compiler_type == "msvc":
            msbuild = _find_msbuild()
            subprocess.call(cmake_command)
            subprocess.call([msbuild, "-p:Configuration=Release", "ALL_BUILD.vcxproj"])
        else:
            raise Exception("Unsupported platform")

        os.chdir("../..")

        build_ext.build_extensions(self)


long_description = ""
if os.path.isfile("README.rst"):
    long_description = open("README.rst", "r").read()


setup(
    name="mozjpeg-lossless-optimization",
    version="1.3.1",
    project_urls={
        "Source Code": "https://github.com/wanadev/mozjpeg-lossless-optimization",
        "Documentation": "https://github.com/wanadev/mozjpeg-lossless-optimization#usage",
        "Changelog": "https://github.com/wanadev/mozjpeg-lossless-optimization#changelog",
        "Issues": "https://github.com/wanadev/mozjpeg-lossless-optimization/issues",
        "Chat": "https://discord.gg/BmUkEdMuFp",
    },
    description="Optimize JPEGs losslessly using MozJPEG",
    url="https://github.com/wanadev/mozjpeg-lossless-optimization",
    license="BSD-3-Clause",
    long_description=long_description,
    keywords="image jpeg mozjpeg jpegtran optimization cffi",
    author="Wanadev",
    author_email="contact@wanadev.fr",
    maintainer="Fabien LOISON",
    packages=find_packages(),
    setup_requires=[
        "cffi>=1.0.0",
    ],
    install_requires=[
        "cffi>=1.0.0",
    ],
    extras_require={
        "dev": [
            "nox",
            "flake8",
            "black",
            "pytest",
        ]
    },
    cffi_modules=[
        "mozjpeg_lossless_optimization/mozjpeg_opti_build.py:ffibuilder",
    ],
    cmdclass={
        "build_ext": CustomBuildExt,
    },
)
